/*
 * This source code (.java) file is Copyright  2000 Artima Software, Inc. All rights reserved. 
 * This file accompanies the Jini Place Service Draft Specification, written by Bill
 * Venners and published on the World Wide Web at:
 * 
 *     http://www.artima.com/jini/cyberspace/DraftSpec.html,
 *
 * This source file may not be copied, modified, or redistributed EXCEPT as allowed
 * by the following statements: From August 11, 2000 through December 31, 2000, you may
 * copy and/or modify these files to test and experiment with the Place API,
 * described in the Jini Place Service Draft Specification. Any bug fixes must be given
 * back to Artima Software, Inc. You may not redistribute this file or any binary (such
 * as .class) files generated from this file. You may not distribute modified versions
 * this files or any binary (such as .class) files generated from modified versions of
 * this file. You may not remove this copyright notice. You may not use this file in
 * printed media without the express permission of Bill Venners. And if that weren't
 * enough, you must destroy all copies of this file, and any binary (such as
 * .class) files generated from this file, by December 31, 2000.
 *
 * ARTIMA SOFTWARE, INC. MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THIS SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
 * BILL VENNERS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY A LICENSEE AS A RESULT
 * OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
*/
package net.artima.place;

import java.util.Locale;
import java.util.ResourceBundle;
import java.io.Serializable;
import java.io.InputStream;
import java.net.URL;

/**
* A bundle of information about a resource represented by a <CODE>Link</CODE>.
*/
public abstract class PropertyFileResourceInfo implements ResourceInfo {

    /**
    * Supported locales.
    */
    private Locale[] supportedLocales;

    /**
    * The ResourceBundle for the default locale.
    */
    private ResourceBundle defaultBundle;

    /**
    * The base name of the properties files containing information returned by
    * the methods of this class. The actual file names are formed by concatenating
    * the base name + [_<language code>] + [_<country code>] + [_<variant>], as
    * described in the ResourceBundle javadoc documentation.
    */
    private String baseName;

    /**
    * The class loader that is able to load the properties files containing information
    * returned by the methods of this class.
    */
    private ClassLoader loader;

    /**
    * The most recently seen default locale.
    */
    private Locale defaultLocale = Locale.getDefault();

    /**
    * Gets the resource bundle for the currently default locale.
    */
    private void getBundle() {

        if (loader == null) {
            loader = getClassLoader();
        }

        defaultLocale = Locale.getDefault();
        defaultBundle = ResourceBundle.getBundle(baseName, defaultLocale, loader);
    }

    /**
    * Construct a PropertyFileResourceInfo.
    *
    * @param baseName Base name of property file.
    * @param loader Class loader that can load the property file as a resource.
    * @param supportedLocales Locales supported by this ResourceInfo
    */
    public PropertyFileResourceInfo(String baseName, Locale[] supportedLocales) {

        this.baseName = baseName;
        this.supportedLocales = supportedLocales;
    }

    /**
    * Returns an array of locales supported by this ResourceInfo.
    * In other words, this ResourceInfo will be able to return user
    * oriented data for each locale in the returned array.
    */
    public Locale[] getSupportedLocales() {

        if (loader == null) {
            loader = getClassLoader();
        }

        return supportedLocales;
    }

    /**
    * Gets the name of this resource, suitable for presentation to users,.
    * under the current locale.
    */
    public String getName() {

        if (defaultBundle == null || !defaultLocale.equals(Locale.getDefault())) {
            getBundle();
        }

        return defaultBundle.getString("name");
    }

    /**
    * Gets the name of this resource, suitable for presentation to users.
    * under the specified locale.
    */
    public String getName(Locale locale) {

        if (loader == null) {
            loader = getClassLoader();
        }

        ResourceBundle bundle = ResourceBundle.getBundle(baseName, locale, loader);
        return bundle.getString("name");
    }

    /**
    * Gets a set of keywords for this resource, suitable for presentation to users,
    * under the current locale. Keywords are used by search engines to index
    * the resource.
    */
    public String[] getKeywords() {

        if (defaultBundle == null || !defaultLocale.equals(Locale.getDefault())) {
            getBundle();
        }
        // Not yet implemented...
        return null;
    }

    /**
    * Gets a set of keywords for this resource, suitable for presentation to users,
    * under the specified locale. Keywords are used by search engines to index
    * the resource.
    */
    public String[] getKeywords(Locale locale) {

        if (loader == null) {
            loader = getClassLoader();
        }

        ResourceBundle bundle = ResourceBundle.getBundle(baseName, locale, loader);
        // Not yet implemented...
        return null;
    }

    /**
    * Gets a short description for this resource, suitable for presentation to users,
    * under the current locale.
    */
    public String getDescription() {

        if (defaultBundle == null || !defaultLocale.equals(Locale.getDefault())) {
            getBundle();
        }
        return defaultBundle.getString("description");
    }

    /**
    * Gets a short description for this resource, suitable for presentation to users,
    * under the specified locale.
    */
    public String getDescription(Locale locale) {

        if (loader == null) {
            loader = getClassLoader();
        }

        ResourceBundle bundle = ResourceBundle.getBundle(baseName, locale, loader);
        return defaultBundle.getString("description");
    }

    /**
    * Gets a URL for the image file of an icon for this resource, suitable for presentation to users,
    * under the current locale. (The image file is likely either a GIF
    * or JPG file, but could be any image file format.) The valid parameter values are the same as for
    * the getIcon method of java.beans.BeanInfo, which for convenience are also declared in this
    * interface: ICON_COLOR_16x16, ICON_COLOR_32x32, ICON_MONO_16x16, ICON_MONO_32x32.
    */
    public URL getIconImageFile(int iconKind) {

        if (defaultBundle == null || !defaultLocale.equals(Locale.getDefault())) {
            getBundle();
        }

        // Get the path name of the icon GIF or JPG file
        String resourceName = defaultBundle.getString(translateIconKindtoString(iconKind));
        URL iconURL = loader.getResource(resourceName);
        return iconURL;
    }

    /**
    * Gets an InputStream for the image file of an icon for this resource, suitable for presentation to users,
    * under the current locale. (The image file is likely either a GIF
    * or JPG file, but could be any image file format.) The valid parameter values are the same as for
    * the getIcon method of java.beans.BeanInfo, which for convenience are also declared in this
    * interface: ICON_COLOR_16x16, ICON_COLOR_32x32, ICON_MONO_16x16, ICON_MONO_32x32.
    */
    public InputStream getIconImageFileAsStream(int iconKind) {

        if (defaultBundle == null || !defaultLocale.equals(Locale.getDefault())) {
            getBundle();
        }

        // Get the path name of the icon GIF or JPG file
        String resourceName = defaultBundle.getString(translateIconKindtoString(iconKind));
        InputStream is = loader.getResourceAsStream(resourceName);
        return is;
    }

    /**
    * Gets a URL for the image file of an icon for this resource, suitable for presentation to users,
    * under the specified locale. (The image file is likely either a GIF
    * or JPG file, but could be any image file format.) The valid parameter values are the same as for
    * the getIcon method of java.beans.BeanInfo, which for convenience are also declared in this
    * interface: ICON_COLOR_16x16, ICON_COLOR_32x32, ICON_MONO_16x16, ICON_MONO_32x32.
    */
    public URL getIconImageFile(Locale locale, int iconKind) {

        if (loader == null) {
            loader = getClassLoader();
        }

        ResourceBundle bundle = ResourceBundle.getBundle(baseName, locale, loader);

        // Get the path name of the icon GIF or JPG file
        String resourceName = bundle.getString(translateIconKindtoString(iconKind));
        URL iconURL = loader.getResource(resourceName);
        return iconURL;
    }

    /**
    * Gets an InputStream for the image file of an icon for this resource, suitable for presentation to users,
    * under the specified locale. (The image file is likely either a GIF
    * or JPG file, but could be any image file format.) The valid parameter values are the same as for
    * the getIcon method of java.beans.BeanInfo, which for convenience are also declared in this
    * interface: ICON_COLOR_16x16, ICON_COLOR_32x32, ICON_MONO_16x16, ICON_MONO_32x32.
    */
    public InputStream getIconImageFileAsStream(Locale locale, int iconKind) {

        if (loader == null) {
            loader = getClassLoader();
        }

        ResourceBundle bundle = ResourceBundle.getBundle(baseName, locale, loader);

        // Get the path name of the icon GIF or JPG file
        String resourceName = bundle.getString(translateIconKindtoString(iconKind));
        InputStream is = loader.getResourceAsStream(resourceName);
        return is;
    }

    /**
    * Overridden in subclasses that are sent in the same JAR file as the
    * property files.
    */
    protected abstract ClassLoader getClassLoader();

    private static String translateIconKindtoString(int iconKind) {

        switch (iconKind) {

        case ResourceInfo.ICON_COLOR_16x16:
            return "ICON_COLOR_16x16";

        case ResourceInfo.ICON_COLOR_32x32:
            return "ICON_COLOR_32x32";

        case ResourceInfo.ICON_MONO_16x16:
            return "ICON_COLOR_16x16";

        case ResourceInfo.ICON_MONO_32x32:
            return "ICON_COLOR_32x32";

        default:
            throw new IllegalArgumentException();
        }
    }
}

